"""DBNet bus integration."""
# https://www.home-assistant.io/integrations/modbus/
# https://www.home-assistant.io/integrations/sensor.modbus/
# /home/tma/work/qore/hass/core/homeassistant/components/citybikes
import logging
import traceback

from dbnet2.dbnet2hub import parse_config, DBNetHub
from typing import Optional, Dict, Any

import homeassistant.helpers.config_validation as cv
from homeassistant.helpers.entity import Entity, async_generate_entity_id
from homeassistant.helpers.entity_component import EntityComponent
from homeassistant import config_entries
import voluptuous as vol
import datetime
from homeassistant.helpers.typing import ConfigType, HomeAssistantType

from homeassistant.const import (
    CONF_NAME,
    EVENT_HOMEASSISTANT_STOP,
)

from .const import (
    ATTR_HUB,
    ATTR_REGISTER,
    ATTR_STATION,
    ATTR_VALUE,
    SERVICE_WRITE_REGISTER,
    SERVICE_WRITE_TIME,
    CONF_STATION_CONFIG,
    CONF_CLIENT_ADDR,
    CONF_EXPIRATION,
    DBNET_DOMAIN as DOMAIN,
    DEFAULT_HUB,
)

_LOGGER = logging.getLogger(__name__)

CONFIG_SCHEMA = vol.Schema({
    DOMAIN: cv.schema_with_slug_keys({
        vol.Optional(CONF_NAME): cv.string,
        vol.Required(CONF_STATION_CONFIG, default=None): cv.isfile,
        vol.Required(CONF_CLIENT_ADDR, default=None): vol.All(vol.Coerce(int), vol.Range(min=1, max=31)),
        vol.Optional(CONF_EXPIRATION, default=1.0): vol.All(vol.Coerce(float), vol.Range(min=0.0, max=9999.9)),
    }),
    }, extra=vol.ALLOW_EXTRA,
)

ENTITY_ID_FORMAT = DOMAIN + ".{}"

"""
CONFIG_SCHEMAx = vol.Schema(
    {DOMAIN: vol.All(cv.ensure_list, [vol.Any(HUB_SCHEMA)])},
    extra=vol.ALLOW_EXTRA,
)
"""

COMPONENTS_WITH_CONFIG_ENTRY = [
    "binary_sensor",
    "climate",
    "light",
    "sensor",
    "switch",
]

def timestamp(value):
    """Timestamp validation."""
    return datetime.datetime.strptime(value, "%Y-%m-%dT%H:%M:%S")


SERVICE_WRITE_REGISTER_SCHEMA = vol.Schema(
    {
        vol.Optional(ATTR_HUB, default=DEFAULT_HUB): cv.string,
        vol.Required(ATTR_REGISTER): cv.string,
        vol.Required(ATTR_VALUE): vol.Any(
            vol.Coerce(float), vol.Coerce(int)
        ),
    }
)

SERVICE_WRITE_TIME_SCHEMA = vol.Schema(
    {
        vol.Optional(ATTR_HUB, default=DEFAULT_HUB): cv.string,
        vol.Required(ATTR_STATION): cv.string,
        vol.Optional(ATTR_VALUE): timestamp,
    }
)


async def async_setup(hass: HomeAssistantType, config: ConfigType) -> bool:
    """Set up Modbus component."""
    print("async_setup", DOMAIN)
    # traceback.print_stack()
    if DOMAIN not in config:
        return True
    if not hass.config_entries.async_entries(DOMAIN):
        # we need get entry_id to display devices but still issue how get config by platform
        hass.async_create_task(
            hass.config_entries.flow.async_init(
                DOMAIN, context={"source": config_entries.SOURCE_IMPORT}, data={}
            )
        )
    hass.data[DOMAIN] = hub_collect = {}
    stations = []
    for hub_id in config[DOMAIN]:
        client_config = config[DOMAIN][hub_id]
        cfg = parse_config(client_config[CONF_STATION_CONFIG])
        hub_stations = {}
        hub_collect[hub_id] = {
            "hub":  DBNetHub(client_config[CONF_CLIENT_ADDR], cfg[0], cfg[1], cfg[2], client_config[CONF_EXPIRATION]),
            "stations": hub_stations
        }
        for station_id in cfg[0]:
            station = DBNetStation(hass, hub_id, client_config.get(CONF_NAME, hub_id), cfg[0][station_id])
            hub_stations[station_id] = station
            stations.append(station)
    component = EntityComponent(_LOGGER, DOMAIN, hass)
    await component.async_add_entities(stations)

    async def stop_dbnet(event):
        """Stop DBNet service."""
        # for client in hub_collect["hub"].values():
        #   client.close()

    async def write_register(service):
        """Write DBNet registers."""
        var = service.data[ATTR_REGISTER]
        value = service.data[ATTR_VALUE]
        client_name = service.data[ATTR_HUB]
        hub_collect[client_name]["hub"].write_reg(var, value)

    async def write_time(service):
        """Write DBNet time."""
        station = service.data[ATTR_STATION]
        client_name = service.data[ATTR_HUB]
        if service.data[ATTR_VALUE] is None:
            hub_collect[client_name]["hub"].write_time(station)
        else:
            date_time = datetime.datetime.fromisoformat(service.data[ATTR_VALUE])
            hub_collect[client_name]["hub"].write_time(station, date_time)

    # register function to gracefully stop DBNet
    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, stop_dbnet)
    # Register services for DBNet
    hass.services.async_register(
        DOMAIN,
        SERVICE_WRITE_REGISTER,
        write_register,
        schema=SERVICE_WRITE_REGISTER_SCHEMA,
    )
    hass.services.async_register(
        DOMAIN,
        SERVICE_WRITE_TIME,
        write_time,
        schema=SERVICE_WRITE_TIME_SCHEMA,
    )
    return True


async def async_setup_entry(hass: HomeAssistantType, entry: ConfigType) -> bool:
    """Set up entry the DbNet2 platform."""
    print('setup_entry', entry)
    # Set up demo platforms with config entry
    for component in COMPONENTS_WITH_CONFIG_ENTRY:
        hass.async_create_task(
            hass.config_entries.async_forward_entry_setup(entry, component)
        )
    return True


class DBNetStation(Entity):
    """DBNet controller class."""

    def __init__(self, hass, hub_name, hub_descr, station, **kwargs):
        """Class constructor."""
        super().__init__(**kwargs)
        self._hub_name = hub_name
        self._hub_descr = hub_descr
        self._station = station
        self._hub = hass.data[DOMAIN][self._hub_name]["hub"]
        self.entity_id = async_generate_entity_id(ENTITY_ID_FORMAT, (self._hub_name + "_" + self._station.name).lower(), hass=hass)

    @property
    def unique_id(self) -> str:
        """Return unique_id for GUI to be modifiable."""
        return self.entity_id

    @property
    def name(self) -> str:
        """Return the name of the entity."""
        return self._station.name

    @property
    def device_info(self) -> Dict[str, Any]:
        """Return device information about controller."""
        result = {
            "identifiers": {(DOMAIN, self._hub_name + "-" + self._station.name)},
            "name": "{station} ({hub})".format(station=self._station.name, hub=self._hub_descr),
            "manufacturer": "Amit",
            "connections": {("addr", "{ip}:{port}/addr={addr}".format(ip=self._station.ip, port=self._station.port, addr=self._station.addr))},
        }
        try:
            sysid = self._hub.get_identity(self._station.name, False)
            result["model"] = "|".join(sysid)
        except RuntimeError:
            pass
        except UnicodeDecodeError:
            pass
        try:
            userid = self._hub.get_identity(self._station.name, True)
            result["sw_version"] = "|".join(userid),
        except RuntimeError:
            pass
        except UnicodeDecodeError:
            pass
        return result

    # https://community.home-assistant.io/t/new-platform-for-fan-passing-config-from-component-to-platform/136619
    # https://programtalk.com/python-examples-amp/homeassistant.helpers.discovery.load_platform/
    # https://github.com/heinoldenhuis/home_assistant_omnik_solar/blob/master/custom_components/omnik/sensor.py
