"""Support for DBNet bus binary sensors."""
import logging
from typing import Optional, Dict, Any

import voluptuous as vol

from homeassistant.components.binary_sensor import (
    DEVICE_CLASSES_SCHEMA,
    PLATFORM_SCHEMA,
    BinarySensorEntity,
    ENTITY_ID_FORMAT,
    DOMAIN,
)
from homeassistant.const import (
    CONF_DEVICE_CLASS,
)

from .const import (
    CONF_REGISTER,
    CONF_STATE_OFF,
    CONF_STATE_ON,

    PROP_OPTIONAL,
    PROP_REQUIRED,
    PROP_ITEM_ONLY,
)

from .helpers import (
    DBNetEntity,
    get_platform_schema,
    dbnet_add_entities,
    val_register,
)

_LOGGER = logging.getLogger(__name__)

ENTITY_PROPS = [
    (CONF_REGISTER, val_register, PROP_REQUIRED + PROP_ITEM_ONLY, None),
    (CONF_DEVICE_CLASS, DEVICE_CLASSES_SCHEMA, PROP_OPTIONAL, None),
    (CONF_STATE_OFF, vol.Coerce(int), PROP_OPTIONAL, 0),
    (CONF_STATE_ON, vol.Coerce(int), PROP_OPTIONAL, 1),
]

PLATFORM_SCHEMA = get_platform_schema(PLATFORM_SCHEMA, ENTITY_PROPS)

async def async_setup_entry(hass, entry, async_add_entities):
    """Set up the DBNetBus binary sensor devices config entry."""
    print("binary sensor async_setup_entry", entry)
    return True


async def async_setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the DbNet2 binary sensors."""
    dbnet_add_entities(hass, config, add_entities, DBNetBinarySensor, ENTITY_ID_FORMAT)


class DBNetBinarySensor(DBNetEntity, BinarySensorEntity):
    """DBNet binary sensor."""

    def __init__(self, hass, id_, config):
        """Initialize the DBNet binary sensor."""
        super().__init__(hass, id_, config)
        self._is_on = None

    @property
    def is_on(self):
        """Return the state of the sensor."""
        return self._is_on

    @property
    def device_class(self) -> Optional[str]:
        """Return the device class of the sensor."""
        return self.prop(CONF_DEVICE_CLASS)

    @property
    def register(self) -> str:
        """Return the register."""
        return self.prop(CONF_REGISTER)

    @property
    def device_state_attributes(self):
        """Return the state attributes of the binary sensor."""
        return {
            'register': self.prop(CONF_REGISTER)
        }

    def update(self):
        """Update the state of the sensor."""
        val = self.read_reg(self.register)
        if not self.available:
            return
        val = int(val)
        if val == self.prop(CONF_STATE_ON):
            self._is_on = True
        elif val == self.prop(CONF_STATE_OFF):
            self._is_on = False
        else:
            _LOGGER.error(
                "Unexpected response from hub %s, var %s, got 0x%2x",
                self._hub_name,
                self.prop(CONF_REGISTER),
                val,
            )
            self.available = False
            return

    @property
    def device_info(self) -> Optional[Dict[str, Any]]:
        """Return device specific attributes."""
        return self.get_device_info(self.prop(CONF_REGISTER))
