"""Support for Generic DBNet bus Thermostats."""
import logging
from typing import Optional, Dict, Any, List

import voluptuous as vol
import homeassistant.helpers.config_validation as cv

from homeassistant.components.climate import (
    PLATFORM_SCHEMA,
    ClimateEntity,
    ENTITY_ID_FORMAT,
    DOMAIN,
)
from homeassistant.components.climate.const import (
    CURRENT_HVAC_IDLE,
    CURRENT_HVAC_HEAT,
    CURRENT_HVAC_COOL,
)
from homeassistant.components.climate.const import (
    HVAC_MODE_AUTO,
    SUPPORT_TARGET_TEMPERATURE,
    SUPPORT_PRESET_MODE,
)
from homeassistant.const import (
    ATTR_TEMPERATURE,
    TEMP_CELSIUS,
    TEMP_FAHRENHEIT,
    CONF_STATE,
)

from .const import (
    CONF_CURRENT_TEMP_REGISTER,
    CONF_TARGET_TEMP_REGISTER,
    CONF_COOLING_REGISTER,
    CONF_HEATING_REGISTER,
    CONF_PRESET_MODE_REGISTER,
    CONF_PRESET_MODES,
    CONF_HVAC_MODE_REGISTER,
    CONF_HVAC_MODES,
    CONF_READ_ONLY,

    CONF_MAX_TEMP,
    CONF_MIN_TEMP,
    CONF_OFFSET,
    CONF_PRECISION,
    CONF_SCALE,
    CONF_STEP,
    CONF_UNIT,

    PROP_OPTIONAL,
    PROP_REQUIRED,
    PROP_PLATFORM_ONLY,
    PROP_ITEM_ONLY,
)

from .helpers import (
    DBNetEntity,
    get_platform_schema,
    dbnet_add_entities,
    val_register,
)

_LOGGER = logging.getLogger(__name__)

MODE_PLAT_PROPS = cv.schema_with_slug_keys(
    vol.Schema({
        vol.Required(CONF_STATE): vol.Coerce(int),
        vol.Optional(CONF_READ_ONLY, default=False): cv.boolean,
    })
)

MODE_ITEM_PROPS = cv.schema_with_slug_keys(
    vol.Schema({
        vol.Optional(CONF_STATE): vol.Coerce(int),
        vol.Optional(CONF_READ_ONLY): cv.boolean,
        vol.Optional(CONF_TARGET_TEMP_REGISTER): val_register,
    })
)

ENTITY_PROPS = [
    (CONF_CURRENT_TEMP_REGISTER, val_register, PROP_REQUIRED + PROP_ITEM_ONLY, None),
    (CONF_TARGET_TEMP_REGISTER, val_register, PROP_OPTIONAL + PROP_ITEM_ONLY, None),
    (CONF_HEATING_REGISTER, val_register, PROP_OPTIONAL + PROP_ITEM_ONLY, None),
    (CONF_COOLING_REGISTER, val_register, PROP_OPTIONAL + PROP_ITEM_ONLY, None),
    (CONF_HVAC_MODE_REGISTER, val_register, PROP_OPTIONAL + PROP_ITEM_ONLY, None),
    (CONF_HVAC_MODES, MODE_PLAT_PROPS, PROP_OPTIONAL + PROP_PLATFORM_ONLY, None),
    (CONF_HVAC_MODES, cv.schema_with_slug_keys(
        vol.Schema({
            vol.Optional(CONF_STATE): vol.Coerce(int),
            vol.Optional(CONF_READ_ONLY): cv.boolean,
        })
    ), PROP_OPTIONAL + PROP_ITEM_ONLY, None),
    (CONF_PRESET_MODE_REGISTER, val_register, PROP_OPTIONAL + PROP_ITEM_ONLY, None),
    (CONF_PRESET_MODES, MODE_PLAT_PROPS, PROP_OPTIONAL + PROP_PLATFORM_ONLY, None),
    (CONF_PRESET_MODES, cv.schema_with_slug_keys(
        vol.Schema({
            vol.Optional(CONF_STATE): vol.Coerce(int),
            vol.Optional(CONF_READ_ONLY): cv.boolean,
            vol.Optional(CONF_TARGET_TEMP_REGISTER): val_register,
        })
    ), PROP_OPTIONAL + PROP_ITEM_ONLY, None),
    (CONF_PRECISION, cv.positive_int, PROP_OPTIONAL, 1),
    (CONF_SCALE, vol.Coerce(float), PROP_OPTIONAL, 1),
    (CONF_OFFSET, vol.Coerce(float), PROP_OPTIONAL, 0),
    (CONF_MAX_TEMP, cv.positive_int, PROP_OPTIONAL, 35),
    (CONF_MIN_TEMP, cv.positive_int, PROP_OPTIONAL, 5),
    (CONF_STEP, vol.Coerce(float), PROP_OPTIONAL, 0.5),
    (CONF_UNIT, cv.string, PROP_OPTIONAL, 'C'),
]

PLATFORM_SCHEMA = get_platform_schema(PLATFORM_SCHEMA, ENTITY_PROPS)

async def async_setup_entry(hass, entry, async_add_entities):
    """Set up the DBNetBus climate devices config entry."""
    print("climate async_setup_entry", entry)
    return True


async def async_setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the DbNet thermostats."""
    dbnet_add_entities(hass, config, add_entities, DBNetThermostat, ENTITY_ID_FORMAT)


class DBNetThermostat(DBNetEntity, ClimateEntity):
    """Representation of a DBNet Thermostat."""

    def __init__(self, hass, id_, config):
        """Initialize the DBNet thermostat."""
        super().__init__(hass, id_, config)
        self._target_temperature = None
        self._current_temperature = None
        self._action = None
        self._hvac_mode = HVAC_MODE_AUTO
        self._preset_mode = None

    @property
    def supported_features(self):
        """Return the list of supported features."""
        result = 0
        if self.prop(CONF_TARGET_TEMP_REGISTER):
            result += SUPPORT_TARGET_TEMPERATURE
        if self.prop(CONF_PRESET_MODE_REGISTER) and self.prop(CONF_PRESET_MODES):
            result += SUPPORT_PRESET_MODE
        return result

    def update(self):
        """Update Target & Current Temperature."""
        val = self.scale(self.read_reg(self.prop(CONF_CURRENT_TEMP_REGISTER)))
        if not self.available:
            return
        self._current_temperature = val

        reg = self._get_target_temp_register()
        if reg:
            val = self.scale(self.read_reg(reg[0]))
            if not self.available:
                return
        self._target_temperature = val

        flag = False
        if self.prop(CONF_HEATING_REGISTER):
            if self.read_reg(self.prop(CONF_HEATING_REGISTER)) != 0:
                self._action = CURRENT_HVAC_HEAT
                flag = True
            else:
                self._action = CURRENT_HVAC_IDLE
        if not flag and self.prop(CONF_COOLING_REGISTER):
            if self.read_reg(self.prop(CONF_COOLING_REGISTER)) != 0:
                self._action = CURRENT_HVAC_COOL
            else:
                self._action = CURRENT_HVAC_IDLE

        if self.prop(CONF_PRESET_MODE_REGISTER) and self.prop(CONF_PRESET_MODES):
            self._preset_mode = self._resolve_mode(self.read_reg(self.prop(CONF_PRESET_MODE_REGISTER)), CONF_PRESET_MODES)

        if self.prop(CONF_HVAC_MODE_REGISTER) and self.prop(CONF_HVAC_MODES):
            self._hvac_mode = self._resolve_mode(self.read_reg(self.prop(CONF_HVAC_MODE_REGISTER)), CONF_HVAC_MODES)

    @property
    def hvac_mode(self) -> str:
        """Return the current HVAC mode."""
        return self._hvac_mode

    @property
    def hvac_modes(self) -> List[str]:
        """Return the possible HVAC modes."""
        if self.prop(CONF_HVAC_MODES):
            ret = list(self.prop(CONF_HVAC_MODES).keys())
        else:
            ret = [self._hvac_mode]
        return ret

    def set_hvac_mode(self, hvac_mode: str) -> None:
        """Set new target hvac mode."""
        if self.prop(CONF_HVAC_MODE_REGISTER):
            self.write_reg(self.prop(CONF_HVAC_MODE_REGISTER), self.prop(CONF_HVAC_MODES)[hvac_mode][CONF_STATE])
        else:
            self._hvac_mode = hvac_mode

    @property
    def preset_mode(self) -> Optional[str]:
        """Return the current preset mode, e.g., home, away, temp.

        Requires SUPPORT_PRESET_MODE.
        """
        return self._preset_mode

    @property
    def preset_modes(self) -> Optional[List[str]]:
        """Return a list of available preset modes.

        Requires SUPPORT_PRESET_MODE.
        """
        if self.prop(CONF_PRESET_MODES):
            ret = list(self.prop(CONF_PRESET_MODES).keys())
        else:
            ret = None
        return ret

    def set_preset_mode(self, preset_mode: str) -> None:
        """Set new preset mode."""
        # do it immediately as target temp is related
        self._preset_mode = preset_mode
        self.write_reg(self.prop(CONF_PRESET_MODE_REGISTER), self.prop(CONF_PRESET_MODES)[preset_mode][CONF_STATE])

    @property
    def current_temperature(self):
        """Return the current temperature."""
        return self._current_temperature

    @property
    def hvac_action(self) -> Optional[str]:
        """Return the current running hvac operation."""
        return self._action

    @property
    def target_temperature(self):
        """Return the target temperature."""
        return self._target_temperature

    @property
    def temperature_unit(self):
        """Return the unit of measurement."""
        return TEMP_FAHRENHEIT if self.prop(CONF_UNIT) == "F" else TEMP_CELSIUS

    @property
    def min_temp(self):
        """Return the minimum temperature."""
        return self.prop(CONF_MIN_TEMP)

    @property
    def max_temp(self):
        """Return the maximum temperature."""
        return self.prop(CONF_MAX_TEMP)

    @property
    def target_temperature_step(self):
        """Return the supported step of target temperature."""
        return self.prop(CONF_STEP)

    def set_temperature(self, **kwargs):
        """Set new target temperature."""
        target_temperature = (kwargs.get(ATTR_TEMPERATURE) - self.prop(CONF_OFFSET) / self.prop(CONF_SCALE))

        reg = self._get_target_temp_register()
        if not reg or reg[1]:
            return
        self.write_reg(reg[0], target_temperature)

    def scale(self, val):
        """Scale value."""
        if val is None:
            return val
        print("scale", val)
        fval = format(
            (self.prop(CONF_SCALE) * val) + self.prop(CONF_OFFSET), f".{self.prop(CONF_PRECISION)}f"
        )
        fval = float(fval)
        return fval

    def _resolve_mode(self, val, conf_modes):
        modes = self.prop(conf_modes)
        for key in modes:
            if modes[key][CONF_STATE] == val:
                return key
        return None

    def _get_target_temp_register(self):
        if not self.prop(CONF_TARGET_TEMP_REGISTER):
            return None
        reg = self.prop(CONF_TARGET_TEMP_REGISTER)
        modes = self.prop(CONF_PRESET_MODES)
        if modes:
            if self._preset_mode in modes:
                mode = modes[self._preset_mode]
                if CONF_TARGET_TEMP_REGISTER in mode:
                    reg = mode[CONF_TARGET_TEMP_REGISTER]
                return (reg, mode[CONF_READ_ONLY])
        return (reg, False)

    @property
    def device_info(self) -> Optional[Dict[str, Any]]:
        """Return device specific attributes."""
        return self.get_device_info(self.prop(CONF_CURRENT_TEMP_REGISTER))
