"""Helpers for DBNet bus."""
# from typing import Callable, Dict, Tuple
from typing import Optional, Dict, Any

import voluptuous as vol
from homeassistant.helpers import config_validation as cv
from homeassistant.helpers.entity import async_generate_entity_id
import re
import copy

from homeassistant.const import (
    CONF_NAME,
)

from .const import (
    CONF_HUB,
    CONF_ITEMS,
    CONF_HIDDEN,
    DEFAULT_HUB,
    DBNET_DOMAIN,

    # PROP_OPTIONAL,
    PROP_REQUIRED,
    PROP_PLATFORM_ONLY,
    PROP_ITEM_ONLY,
)
"""
    Define properties for entity as list of tupples
    [(name, type, required, default), ...]
ENTITY_PROPS = [
    (CONF_NAME, cv.string, PROP_REQUIRED, None),
    (CONF_REGISTER, cv.string, PROP_REQUIRED, None),
    (CONF_OFFSET, vol.Coerce(float), PROP_OPTIONAL, 1),
]
"""


def val_register(value: Any) -> str:
    """Coerce a value to register format."""
    value = str(value)
    value = value.replace(' ', '')
    ok = False
    m = re.match(r'^(@?)([A-Za-z_0-9]+)(.*)$', value)
    if m:
        # alias
        if m[1] == '@':
            if not re.match(r'^[0-9]', m[2]) and m[3] == '':
                # no wid number and no suffix
                ok = True
        else:
            if not re.match(r'^[0-9][A-Za-z_]+', m[2]):
                # name not starting with digit and having a alpha char
                if m[3] == '':
                    ok = True
                else:
                    m = re.match(r'^(\[[0-9]+,[0-9]+\])?(.*)$', m[3])
                    if m:
                        if m[2] == '':
                            ok = True
                        else:
                            m = re.match(r'^\.[0-9]+(.*)$', m[2])
                            if m:
                                if m[1] == '':
                                    ok = True
                                else:
                                    if re.match(r'^:[0-9]+$', m[1]):
                                        ok = True
    if not ok:
        raise ValueError('Wrong register format: {val}'.format(val=value))
    return value


def get_platform_schema(platform_schema, props):
    """Get PLATFORM_SCHEMA from simplified prop definition."""
    plat_opts = {
        vol.Optional(CONF_HUB, default=DEFAULT_HUB): cv.string,
    }
    for prop in props:
        if (prop[2] & PROP_ITEM_ONLY) == 0:
            if (prop[2] & PROP_REQUIRED) == 0:
                # not required
                if not prop[3] is None:
                    plat_opts[vol.Optional(prop[0], default=prop[3])] = prop[1]
                else:
                    plat_opts[vol.Optional(prop[0])] = prop[1]
            else:
                plat_opts[vol.Required(prop[0])] = prop[1]

    item_opts = {
        vol.Optional(CONF_NAME): cv.string,
        vol.Optional(CONF_HIDDEN): cv.boolean,
    }
    for prop in props:
        if (prop[2] & PROP_PLATFORM_ONLY) == 0:
            if (prop[2] & PROP_REQUIRED) == 0:
                # not required, potential defaul is supposed from platform
                item_opts[vol.Optional(prop[0])] = prop[1]
            else:
                item_opts[vol.Required(prop[0])] = prop[1]
        plat_opts[CONF_ITEMS] = cv.schema_with_slug_keys(
            item_opts
        )
    platform_schema = platform_schema.extend(plat_opts)
    return platform_schema


def dbnet_add_entities(hass, config, add_entities, cls, entity_id_format, **kwargs):
    """Create entities from configuration."""
    items = config.get(CONF_ITEMS, {})
    entities = []
    del config[CONF_ITEMS]
    for id_ in items:
        cfg = copy.deepcopy(config)
        entry = items[id_]
        # merge data with Dict of Dict merge support
        for key in entry:
            if key not in cfg:
                cfg[key] = entry[key]
            else:
                if isinstance(entry[key], Dict):
                    for k2 in entry[key]:
                        if k2 in cfg[key]:
                            cfg[key][k2].update(entry[key][k2])
                        else:
                            cfg[key][k2] = entry[key][k2]
                else:
                    cfg[key] = entry[key]
        entity = cls(hass, id_, cfg, **kwargs)
        entity.entity_id = async_generate_entity_id(entity_id_format, id_, hass=hass)
        entities.append(entity)
    add_entities(entities)


class DBNetEntity:
    """DBNet helper class."""

    def __init__(self, hass, id_, config, **kwargs):
        """Class constructor."""
        super().__init__(**kwargs)
        self._hub_name = config[CONF_HUB]
        self._hub = hass.data[DBNET_DOMAIN][self._hub_name]["hub"]
        self._stations = hass.data[DBNET_DOMAIN][self._hub_name]["stations"]
        self._id = id_
        self._name = config.get(CONF_NAME, id_)
        self._hidden = config.get(CONF_HIDDEN, False)
        for key in [CONF_HUB, CONF_NAME, CONF_HIDDEN]:
            if key in config:
                del config[key]
        self._props = config
        self._available = True
        self.entity_id = None

    def get_device_info(self, register):
        """Get entity specific device info for station with register."""
        var = self._hub.resolve_variable(register);
        if not var or not var[0] in self._stations:
            return {}
        result = self._stations[var[0]].device_info
        return result

    @property
    def name(self):
        """Return the name of the sensor."""
        s = self._name
        matches = re.finditer(r'(\$[a-zA-Z_0-9]+\[[0-9]+\])', s)
        for itr in matches:
            m = re.match(r'^\$(.+)\[([0-9]+)\]$', itr.group(1))
            try:
                # s2 = self._hub.read_string(m[1], int(m[2]), 1)
                # get init values to show values even station is disconnected
                v = self._hub.resolve_variable(m[1])
                strs = v[1].get_init_strings()
                s = s.replace(itr.group(0), strs[int(m[2])])
            except Exception as ex:
                print(ex)
                pass
        self._name = s
        return s

    @property
    def unique_id(self) -> str:
        """Return unique_id for GUI to be modifiable."""
        return self.entity_id + "." + self._hub_name

    @property
    def available(self) -> bool:
        """Return True if entity is available."""
        return self._available

    @available.setter
    def available(self, value):
        self._available = value

    @property
    def hidden(self) -> bool:
        """Return True if the entity should be hidden from UIs."""
        return self._hidden

    def prop(self, name):
        """Return private attribute by CONF_xxx."""
        if name in self._props:
            ret = self._props[name]
        else:
            ret = None
        return ret

    def read_reg(self, name):
        """Read register, handle exception and set available."""
        try:
            print('read_reg: ', name, name)
            val = self._hub.read_reg(name)[1]
            print('val', val)
        except:
            self._available = False
            return
        self._available = True
        return val

    def write_reg(self, name, value):
        """Write reg using the hub."""
        try:
            print('write_reg: ', name, name, value)
            self._hub.write_reg(name, value)
            print('write_reg done')
        except:
            self._available = False
            return

        self._available = True
