"""Support for DBNet bus lights."""
import logging
from typing import Optional, Dict, Any

import voluptuous as vol

from homeassistant.components.light import (
    PLATFORM_SCHEMA,
    LightEntity,
    ENTITY_ID_FORMAT,
    DOMAIN,
)
from homeassistant.const import (
    STATE_ON,
    STATE_OFF,
)
from homeassistant.helpers import config_validation as cv
from homeassistant.helpers.restore_state import RestoreEntity

from .const import (
    CONF_REGISTER,
    CONF_STATE_OFF,
    CONF_STATE_ON,
    CONF_STATE_STRICT,
    CONF_VERIFY_STATE,
    STATE_ALL,

    PROP_OPTIONAL,
    PROP_REQUIRED,
    PROP_ITEM_ONLY,
)

from .helpers import (
    DBNetEntity,
    get_platform_schema,
    dbnet_add_entities,
    val_register,
)

_LOGGER = logging.getLogger(__name__)

ENTITY_PROPS = [
    (CONF_REGISTER, val_register, PROP_REQUIRED + PROP_ITEM_ONLY, None),
    (CONF_STATE_OFF, vol.Coerce(int), PROP_OPTIONAL, 0),
    (CONF_STATE_ON, vol.Coerce(int), PROP_OPTIONAL, 1),
    (CONF_STATE_STRICT, vol.All(vol.Lower, vol.In([STATE_ON, STATE_OFF, STATE_ALL])), PROP_OPTIONAL, STATE_ALL),
    (CONF_VERIFY_STATE, cv.boolean, PROP_OPTIONAL, True),
]

PLATFORM_SCHEMA = get_platform_schema(PLATFORM_SCHEMA, ENTITY_PROPS)

async def async_setup_entry(hass, entry, async_add_entities):
    """Set up the DBNetBus light devices config entry."""
    print("light async_setup_entry", entry)
    return True

async def async_setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the DbNet2 lights."""
    dbnet_add_entities(hass, config, add_entities, DBNetLight, ENTITY_ID_FORMAT)


class DBNetLight(DBNetEntity, LightEntity, RestoreEntity):
    """DBNet light."""

    def __init__(self, hass, id_, config):
        """Initialize the DBNet light."""
        super().__init__(hass, id_, config)
        self._is_on = None

    async def async_added_to_hass(self):
        """Handle entity which will be added."""
        state = await self.async_get_last_state()
        if not state:
            return
        self._is_on = state.state == STATE_ON

    @property
    def is_on(self):
        """Return the state of the light."""
        return self._is_on

    @property
    def device_state_attributes(self):
        """Return the state attributes of the light."""
        return {
            'register': self.prop(CONF_REGISTER)
        }

    @property
    def register(self) -> str:
        """Return the register."""
        return self.prop(CONF_REGISTER)

    def turn_on(self, **kwargs):
        """Set light on."""
        self.write_reg(self.register, self.prop(CONF_STATE_ON))
        if self.available and not self.prop(CONF_VERIFY_STATE):
            self._is_on = True

    def turn_off(self, **kwargs):
        """Set light off."""
        self.write_reg(self.register, self.prop(CONF_STATE_OFF))
        if self.available and not self.prop(CONF_VERIFY_STATE):
            self._is_on = False

    def update(self):
        """Update the state of the light."""
        val = self.read_reg(self.register)
        if not self.available:
            return
        val = int(val)
        if val == self.prop(CONF_STATE_ON):
            self._is_on = True
        elif val == self.prop(CONF_STATE_OFF):
            self._is_on = False
        elif self.prop(CONF_STATE_STRICT) == STATE_ON:
            self._is_on = False
        elif self.prop(CONF_STATE_STRICT) == STATE_OFF:
            self._is_on = True
        else:
            _LOGGER.error(
                "Unexpected response from hub %s, var %s, got 0x%2x",
                self._hub_name,
                self.prop(CONF_REGISTER),
                val,
            )
            self.available = False
            return

    @property
    def device_info(self) -> Optional[Dict[str, Any]]:
        """Return device specific attributes."""
        return self.get_device_info(self.prop(CONF_REGISTER))
