"""Support for DBNet bus sensors."""
import logging
from typing import Optional, Union, Any, Dict

import voluptuous as vol

from homeassistant.components.sensor import (
    DEVICE_CLASSES_SCHEMA,
    PLATFORM_SCHEMA,
    ENTITY_ID_FORMAT,
    DOMAIN,
)
from homeassistant.const import (
    CONF_DEVICE_CLASS,
    CONF_UNIT_OF_MEASUREMENT,
)

from homeassistant.helpers import config_validation as cv
from homeassistant.helpers.restore_state import RestoreEntity

from .const import (
    CONF_REGISTER,
    CONF_SCALE,
    CONF_OFFSET,
    CONF_PRECISION,

    PROP_OPTIONAL,
    PROP_REQUIRED,
    PROP_ITEM_ONLY,
)

from .helpers import (
    DBNetEntity,
    get_platform_schema,
    dbnet_add_entities,
    val_register,
)

_LOGGER = logging.getLogger(__name__)


def number(value: Any) -> Union[int, float]:
    """Coerce a value to number without losing precision."""
    if isinstance(value, int):
        return value

    if isinstance(value, str):
        try:
            value = int(value)
            return value
        except (TypeError, ValueError):
            pass

    try:
        value = float(value)
        return value
    except (TypeError, ValueError):
        raise vol.Invalid(f"invalid number {value}")


ENTITY_PROPS = [
    (CONF_REGISTER, val_register, PROP_REQUIRED + PROP_ITEM_ONLY, None),
    (CONF_DEVICE_CLASS, DEVICE_CLASSES_SCHEMA, PROP_OPTIONAL, None),
    (CONF_SCALE, number, PROP_OPTIONAL, 1),
    (CONF_OFFSET, number, PROP_OPTIONAL, 0),
    (CONF_PRECISION, cv.positive_int, PROP_OPTIONAL, 0),
    (CONF_UNIT_OF_MEASUREMENT, cv.string, PROP_OPTIONAL, None),
]

PLATFORM_SCHEMA = get_platform_schema(PLATFORM_SCHEMA, ENTITY_PROPS)


async def async_setup_entry(hass, entry, async_add_entities):
    """Set up the DBNetBus sensor devices config entry."""
    print("sensor async_setup_entry", entry)
    return True


async def async_setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the DbNet2 sensors."""
    dbnet_add_entities(hass, config, add_entities, DBNetSensor, ENTITY_ID_FORMAT)


class DBNetSensor(DBNetEntity, RestoreEntity):
    """DBNet sensor."""

    def __init__(self, hass, id_, config):
        """Initialize the DBNet sensor."""
        super().__init__(hass, id_, config)
        self._value = None

    async def async_added_to_hass(self):
        """Handle entity which will be added."""
        state = await self.async_get_last_state()
        if not state:
            return
        self._value = state.state

    @property
    def state(self):
        """Return the state of the sensor."""
        return self._value

    @property
    def device_class(self) -> Optional[str]:
        """Return the device class of the sensor."""
        return self.prop(CONF_DEVICE_CLASS)

    @property
    def unit_of_measurement(self):
        """Return the unit of measurement."""
        return self.prop(CONF_UNIT_OF_MEASUREMENT)

    @property
    def device_state_attributes(self):
        """Return the state attributes of the sensor."""
        return {
            'register': self.prop(CONF_REGISTER)
        }

    @property
    def register(self) -> str:
        """Return the register."""
        return self.prop(CONF_REGISTER)

    def update(self):
        """Update the state of the sensor."""
        val = self.read_reg(self.register)
        if not self.available:
            return
        val = self.prop(CONF_SCALE) * val + self.prop(CONF_OFFSET)
        if isinstance(val, int):
            self._value = str(val)
            if self.prop(CONF_PRECISION) > 0:
                self._value += "." + "0" * self.prop(CONF_PRECISION)
        else:
            self._value = f"{val:.{self.prop(CONF_PRECISION)}f}"

    @property
    def device_info(self) -> Optional[Dict[str, Any]]:
        """Return device specific attributes."""
        return self.get_device_info(self.prop(CONF_REGISTER))
